<?php

/**
 * @file
 * This file contains CCK widget related functionality.
 *
 * Uses content.module to store the aid and field specific metadata,
 * and Drupal's files table to store the actual file data.
 */

/**
 * Implements hook_widget_settings().
 */
function embridge_field_widget_settings_form($widget) {
  $form = array();
  // Convert the extensions list to be a human-friendly comma-separated list.
  $extensions = is_string($widget['file_extensions']) ? $widget['file_extensions'] : 'txt';
  $form['file_extensions'] = array(
    '#type' => 'textfield',
    '#title' => t('Permitted upload file extensions'),
    '#default_value' => $extensions,
    '#size' => 64,
    '#maxlength' => 512,
    '#description' => t('Extensions a user can upload to this field. Separate extensions with a space and do not include the leading dot. Leaving this blank will allow users to upload a file with any extension.'),
    '#element_validate' => array('embridge_field_widget_settings_extensions_validate'),
    '#pre_render' => array('embridge_field_widget_settings_extensions_value'),
    '#weight' => 1,
  );

  $catalogs = variable_get('embridge_catalogs', array());
  $form['asset_catalog'] = array(
    '#type' => 'select',
    '#title' => t('Entermedia Catalog'),
    '#default_value' => !empty($widget['asset_catalog']) ? $widget['asset_catalog'] : ENTERMEDIA_ALLOWED_ASSET_CATALOG_DEFAULT,
    '#options' => $catalogs,
    '#required' => TRUE,
    '#description' => t('Select Entermedia catalog for asset.'),
  );

  $form['asset_path'] = array(
    '#type' => 'textfield',
    '#title' => t('Asset Path'),
    '#default_value' => !empty($widget['asset_path']) ? $widget['asset_path'] : '',
    '#required' => FALSE,
    '#description' => t('Used to create subfolders during asset upload. Use token [yyyy] for current year and [mm] for current month.'),
  );

  $form['progress_indicator'] = array(
    '#type' => 'radios',
    '#title' => t('Progress indicator'),
    '#options' => array(
      'bar' => t('Bar with progress meter'),
      'throbber' => t('Throbber'),
    ),
    '#default_value' => empty($widget['progress_indicator']) ? 'bar' : $widget['progress_indicator'],
    '#description' => t('Your server supports upload progress capabilities. The "throbber" display does not indicate progress but takes up less room on the form, you may want to use it if you\'ll only be uploading small files or if experiencing problems with the progress bar.'),
    '#weight' => 5,
    '#access' => embridge_field_progress_implementation(),
  );

  $form['max_filesize'] = array(
    '#type' => 'fieldset',
    '#title' => t('File size restrictions'),
    '#description' => t('Limits for the size of files that a user can upload. Note that these settings only apply to newly uploaded files, whereas existing files are not affected.'),
    '#weight' => 6,
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
  );
  $form['max_filesize']['max_filesize_per_file'] = array(
    '#type' => 'textfield',
    '#title' => t('Maximum upload size per file'),
    '#default_value' => is_string($widget['max_filesize_per_file']) ? $widget['max_filesize_per_file'] : '',
    '#description' => t('Specify the size limit that applies to each file separately. Enter a value like "512" (bytes), "80K" (kilobytes) or "50M" (megabytes) in order to restrict the allowed file size. If you leave this empty the file sizes will be limited only by PHP\'s maximum post and file upload sizes (current limit <strong>%limit</strong>).', array('%limit' => format_size(file_upload_max_size()))),
    '#element_validate' => array('embridge_field_widget_settings_max_filesize_per_file_validate'),
  );
  $form['max_filesize']['max_filesize_per_node'] = array(
    '#type' => 'textfield',
    '#title' => t('Maximum upload size per node'),
    '#default_value' => is_string($widget['max_filesize_per_node']) ? $widget['max_filesize_per_node'] : '',
    '#description' => t('Specify the total size limit for all files in field on a given node. Enter a value like "512" (bytes), "80K" (kilobytes) or "50M" (megabytes) in order to restrict the total size of a node. Leave this empty if there should be no size restriction.'),
    '#element_validate' => array('embridge_field_widget_settings_max_filesize_per_node_validate'),
  );

  return $form;
}

/**
 * Implements hook_widget_settings().
 */
function embridge_field_widget_settings_save($widget) {
  return array(
    'file_extensions',
    'progress_indicator',
    'asset_catalog',
    'asset_path',
    'max_filesize_per_file',
    'max_filesize_per_node',
  );
}

/**
 * A FAPI #pre_render() function to set a cosmetic default value for extensions.
 */
function embridge_field_widget_settings_extensions_value($element) {
  $element['#value'] = implode(', ', array_filter(explode(' ', str_replace(',', ' ', $element['#value']))));
  return $element;
}

/**
 * A FAPI #element_validate callback to strip commas from extension lists.
 */
function embridge_field_widget_settings_extensions_validate($element, &$form_state) {
  // Remove commas and leading dots from file extensions.
  $value = str_replace(',', ' ', $element['#value']);
  $value = str_replace(' .', ' ', $value);
  $value = array_filter(explode(' ', $value));
  $value = implode(' ', $value);
  form_set_value($element, $value, $form_state);
}

/**
 * Validation function to validate file size (per file).
 *
 */
function embridge_field_widget_settings_max_filesize_per_file_validate($element, &$form_state) {
  if (empty($form_state['values']['max_filesize_per_file'])) {
    // Empty means no size restrictions, so don't throw an error.
    return;
  }
  elseif (!is_numeric(parse_size($form_state['values']['max_filesize_per_file']))) {
    form_error($element, t('The "@field" option must contain a valid value. You can either leave the text field empty or enter a string like "512" (bytes), "80K" (kilobytes) or "50M" (megabytes).', array('@field' => t('Maximum upload size per file'))));
  }
}

/**
 * Validation function to validate file size (per node).
 *
 */
function embridge_field_widget_settings_max_filesize_per_node_validate($element, &$form_state) {
  if (empty($form_state['values']['max_filesize_per_node'])) {
    // Empty means no size restrictions, so don't throw an error.
    return;
  }
  elseif (!is_numeric(parse_size($form_state['values']['max_filesize_per_node']))) {
    form_error($element, t('The "@field" option must contain a valid value. You can either leave the text field empty or enter a string like "512" (bytes), "80K" (kilobytes) or "50M" (megabytes).', array('@field' => t('Maximum upload size per node'))));
  }
}

/**
 * Determine the widget's files directory
 *
 * @param Array $field
 *   A CCK field array.
 * @param Object $account
 *   The user account object to calculate the file path for.
 *
 * @return String
 *   The files directory path, with any tokens replaced.
 */
function embridge_field_widget_file_path($field, $account = NULL) {
  $account = isset($account) ? $account : $GLOBALS['user'];
  $dest = $field['widget']['file_path'];
  // Replace user level tokens.
  // Node level tokens require a lot of complexity like temporary storage
  // locations when values don't exist. See the embridge_field_paths
  // module.
  if (module_exists('token')) {
    $dest = token_replace($dest, 'user', $account);
  }
  // Replace nasty characters in the path if possible.
  if (module_exists('transliteration')) {
    module_load_include('inc', 'transliteration');
    $dest_array = array_filter(explode('/', $dest));
    foreach ($dest_array as $key => $directory) {
      $dest_array[$key] = transliteration_clean_filename($directory);
    }
    $dest = implode('/', $dest_array);
  }

  return file_directory_path() . '/' . $dest;
}

/**
 * Given a FAPI element, save any files that may have been uploaded into it.
 *
 * This function should only be called during validate, submit, or
 * value_callback functions.
 *
 * @param Array $element
 *   The FAPI element whose values are being saved.
 */
function embridge_field_save_upload($element) {
  $upload_name = implode('_', $element['#array_parents']);
  $field = content_fields($element['#field_name'], $element['#type_name']);
  $catalog_id = $field['widget']['asset_catalog'];
  if (empty($_FILES['files']['name'][$upload_name])) {
    return 0;
  }

  if (!$file = embridge_field_file_save_upload($field, $upload_name, $element['#upload_validators'], $dest)) {
    watchdog('embridge_field', 'The file upload failed. %upload', array('%upload' => $upload_name));
    form_set_error($upload_name, t('The file in the @field field was unable to be uploaded.', array('@field' => $element['#title'])));
    return 0;
  }
  return $file['aid'];
}

/**
 * The #value_callback for the embridge_field_widget type element.
 */
function embridge_field_widget_value($element, $edit = FALSE) {
  // TODO: Place for upload.
  if (!$edit) {
    $file = embridge_field_file_load($element['#default_value']['aid'], $element['#field_name'], $element['#type_name']);
    $item = $element['#default_value'];
  }
  else {
    $item = $edit;
    $field = content_fields($element['#field_name'], $element['#type_name']);

    // Uploads take priority over value of aid text field.
    if ($aid = embridge_field_save_upload($element)) {
      $item['aid'] = $aid;
    }
    elseif (isset($element['#embridge_field_value_callback'])) {
      // Check for #embridge_field_value_callback values.
      // Because FAPI does not allow multiple #value_callback values like it
      // does for #element_validate and #process, this fills the missing
      // functionality to allow embridge_field to be extended purely
      // through FAPI.
      foreach ($element['#embridge_field_value_callback'] as $callback) {
        $callback($element, $item);
      }
    }

    // Set embed code to entermedia asset if exists.
    if ($item['aid'] && isset($edit['embedcode'])) {
      $catalog_id = $field['widget']['asset_catalog'];
      embridge_field_file_update_embed_code($item['aid'], $catalog_id, $edit['embedcode']);
    }

    // Load file if the aid has changed so that it can be saved by CCK.
    $file = embridge_field_file_load($item['aid'], $element['#field_name'], $element['#type_name']);

    // If the file entry doesn't exist, don't save anything.
    if (empty($file)) {
      $item = array();
    }

    // Checkboxes loose their value when empty.
    // If the list field is present make sure its unchecked value is saved.
    if (!empty($field['list_field']) && empty($edit['list'])) {
      $item['list'] = 0;
    }
  }
  // Merge file and item data so it is available to all widgets.
  if (isset($item['data']) && isset($file['data'])) {
    $file['data'] = array_merge($item['data'], $file['data']);
  }
  $item = array_merge($item, $file);

  return $item;
}

/**
 * An element #process callback for the embridge_field_widget field type.
 *
 * Expands the embridge_field_widget type to include the upload field,
 * upload and remove buttons, and the description field.
 */
function embridge_field_widget_process($element, $edit, &$form_state, $form) {
  static $settings_added;
  $item = $element['#value'];
  $field_name = $element['#field_name'];
  $delta = $element['#delta'];
  $element['#theme'] = 'embridge_field_widget_item';

  $field = $form['#field_info'][$field_name];

  // The widget is being presented, so apply the JavaScript.
  drupal_add_js(drupal_get_path('module', 'embridge_field') . '/embridge_field.js');
  if (!isset($settings_added[$field_name]) && isset($element['#upload_validators']['embridge_field_validate_extensions'])) {
    $settings_added[$field_name] = TRUE;
    $settings = array(
      'entermediaasset' => array(
        $field_name => $element['#upload_validators']['embridge_field_validate_extensions'][0],
      ),
    );
    drupal_add_js($settings, 'setting');
  }

  // Title is not necessary for each individual field.
  if ($field['multiple'] > 0) {
    unset($element['#title']);
  }

  // Set up the buttons first since we need to check if they were clicked.
  $element['embridge_field_upload'] = array(
    '#type' => 'submit',
    '#value' => t('Upload'),
    '#submit' => array('node_form_submit_build_node'),
    // With JavaScript.
    '#ahah' => array(
      'path' => 'embridge_field/ahah/' . $element['#type_name'] . '/' . $element['#field_name'] . '/' . $element['#delta'],
      'wrapper' => $element['#id'] . '-ahah-wrapper',
      'method' => 'replace',
      'effect' => 'fade',
    ),
    '#field_name' => $element['#field_name'],
    '#delta' => $element['#delta'],
    '#type_name' => $element['#type_name'],
    '#upload_validators' => $element['#upload_validators'],
    '#weight' => 100,
    '#post' => $element['#post'],
  );
  $element['embridge_field_remove'] = array(
    // With default CCK edit forms, $element['#parents'] is
    // array($element['#field_name'], $element['#delta']).
    // However, if some module (for example, flexifield) places our widget
    // deeper in the tree, we want to use that information in constructing
    // the button name.
    '#name' => implode('_', $element['#parents']) . '_embridge_field_remove',
    '#type' => 'submit',
    '#value' => t('Remove'),
    '#submit' => array('node_form_submit_build_node'),
    // With JavaScript.
    '#ahah' => array(
      'path' => 'embridge_field/ahah/' . $element['#type_name'] . '/' . $element['#field_name'] . '/' . $element['#delta'],
      'wrapper' => $element['#id'] . '-ahah-wrapper',
      'method' => 'replace',
      'effect' => 'fade',
    ),
    '#field_name' => $element['#field_name'],
    '#delta' => $element['#delta'],
    '#weight' => 101,
    '#post' => $element['#post'],
  );

  // Because the output of this field changes depending on the button clicked,
  // we need to ask FAPI immediately if the remove button was clicked.
  // It's not good that we call this private function, but
  // $form_state['clicked_button'] is only available after this #process
  // callback is finished.
  if (_form_button_was_clicked($element['embridge_field_remove'])) {
    // Delete the file if it is currently unused. Note that
    // embridge_field_file_delete()
    // does a reference check in addition to our basic status check.
    if (isset($edit['aid'])) {
      $removed_file = embridge_field_file_load($edit['aid'], $element['#field_name'], $element['#type_name']);
      if ($removed_file['status'] == 0) {
        embridge_field_file_delete($removed_file);
      }
    }
    $item = array(
      'aid' => 0,
      'list' => $field['list_default'],
      'data' => array('description' => ''),
    );
  }
  $field = content_fields($element['#field_name'], $element['#type_name']);
  $catalog_id = $field['widget']['asset_catalog'];
  $element['embridge_field_attach'] = array(
    '#name' => implode('_', $element['#parents']) . '_embridge_field_attach',
    '#type' => 'submit',
    '#value' => t('Attach'),
    '#submit' => array('node_form_submit_build_node'),
    // With JavaScript.
    '#ahah' => array(
      'path' => 'embridge_field/ahah/' . $element['#type_name'] . '/' . $element['#field_name'] . '/' . $element['#delta'],
      'wrapper' => $element['#id'] . '-ahah-wrapper',
      'method' => 'replace',
      'effect' => 'fade',
    ),
    '#field_name' => $element['#field_name'],
    '#delta' => $element['#delta'],
    '#weight' => 104,
    '#post' => $element['#post'],
    '#access' => !empty($item['aid']) && (stripos($catalog_id, 'photo') === FALSE),
  );
  // Attach thumbnail for the asset.
  if (!empty($item['aid']) && $_FILES['thumbnails']['name']) {
    embridge_field_file_attach_thumbnail($item['aid'], $element['#field_name'], $element['#type_name'], $element['#array_parents']);
  }

  $element['embridge_field_upload']['#access'] = empty($item['aid']);
  $element['embridge_field_remove']['#access'] = !empty($item['aid']);

  // Add progress bar support to the upload if possible.
  $progress_indicator = isset($field['widget']['progress_indicator']) ? $field['widget']['progress_indicator'] : 'bar';
  if ($progress_indicator != 'throbber' && $implementation = embridge_field_progress_implementation()) {
    $upload_progress_key = md5(mt_rand());

    if ($implementation == 'uploadprogress') {
      $element['UPLOAD_IDENTIFIER'] = array(
        '#type' => 'hidden',
        '#value' => $upload_progress_key,
        '#attributes' => array('class' => 'embridge-field-progress'),
      );
    }
    elseif ($implementation == 'apc') {
      $element['APC_UPLOAD_PROGRESS'] = array(
        '#type' => 'hidden',
        '#value' => $upload_progress_key,
        '#attributes' => array('class' => 'embridge-field-progress'),
      );
    }

    // Add the upload progress callback.
    $element['embridge_field_upload']['#ahah']['progress']['type'] = 'bar';
    $element['embridge_field_upload']['#ahah']['progress']['path'] = 'embridge_field/progress/' . $upload_progress_key;
  }

  // Set the aid.
  $element['aid'] = array(
    '#type' => 'hidden',
    '#value' => $item['aid'],
  );

  if ($item['aid'] != 0) {
    $element['preview'] = array(
      '#type' => 'markup',
      '#value' => theme('embridge_field_widget_preview', $item),
    );
  }

  // Grant access to temporary files.
  if ($item['aid'] && isset($item['status']) && $item['status'] == 0 && variable_get('file_downloads', FILE_DOWNLOADS_PUBLIC) == FILE_DOWNLOADS_PRIVATE) {
    $_SESSION['embridge_field_access'][] = $item['aid'];
  }

  // placeholder.. will be serialized into the data column. this is a place
  // for widgets to put additional data.
  $element['data'] = array(
    '#tree' => 'true',
    '#access' => !empty($item['aid']),
  );

  if (!empty($field['description_field'])) {
    $element['data']['description'] = array(
      '#type' => 'textfield',
      '#title' => t('Description'),
      '#value' => isset($item['data']['description']) ? $item['data']['description'] : '',
      '#type' => variable_get('embridge_field_description_type', 'textfield'),
      '#maxlength' => variable_get('embridge_field_description_length', 128),
    );
  }

  if (!empty($field['list_field'])) {
    $element['list'] = array(
      '#type' => empty($item['aid']) ? 'hidden' : 'checkbox',
      '#title' => t('List'),
      '#value' => isset($item['list']) && !empty($item['aid']) ? $item['list'] : $field['list_default'],
      '#attributes' => array('class' => 'embridge-field-list'),
    );
  }
  else {
    $element['list'] = array(
      '#type' => 'hidden',
      '#value' => '1',
    );
  }

  foreach ($element['#upload_validators'] as $callback => $arguments) {
    $help_func = $callback . '_help';
    if (function_exists($help_func)) {
      $desc[] = call_user_func_array($help_func, $arguments);
    }
  }

  $element['upload'] = array(
    '#name' => 'files[' . implode('_', $element['#array_parents']) . ']',
    '#type' => 'file',
    '#description' => implode('<br />', $desc),
    '#size' => 22,
    '#access' => empty($item['aid']),
  );

  $element['#attributes']['id'] = $element['#id'] . '-ahah-wrapper';
  $element['#prefix'] = '<div ' . drupal_attributes($element['#attributes']) . '>';
  $element['#suffix'] = '</div>';

  $element['attach'] = array(
    '#name' => 'thumbnails[' . implode('_', $element['#array_parents']) . ']',
    '#type' => 'file',
    '#description' => t('Attach teaser image to the asset as thumbnail and preview display.'),
    '#size' => 22,
    '#access' => (stripos($catalog_id, 'photo') === FALSE && stripos($catalog_id, 'interactive') === FALSE),
    '#weight' => 103,
    '#title' => t('Teaser Image'),
  );
  // Add embed code to entermedia asset.
  $element['embedcode'] = array(
    '#type' => 'textarea',
    '#title' => t('Embed Code'),
    // Embed code is required if it is enabled.
    '#required' => (stripos($catalog_id, 'photo') === FALSE && stripos($catalog_id, 'audio') === FALSE),
    '#value' => isset($item['embedcode']) ? $item['embedcode'] : '',
    '#rows' => 5,
    '#weight' => 104,
    '#access' => (stripos($catalog_id, 'photo') === FALSE && stripos($catalog_id, 'audio') === FALSE),
  );

  return $element;
}

/**
 * An #element_validate callback for the embridge_field_widget field.
 */
function embridge_field_widget_validate(&$element, &$form_state) {
  // If referencing an existing file, only allow if there are existing
  // references. This prevents unmanaged files
  // (outside of embridge_field) from being deleted if this node were
  // to be deleted.
  if (!empty($element['aid']['#value'])) {
    $field = content_fields($element['#field_name'], $element['#type_name']);
    if ($file = embridge_field_file_load($element['aid']['#value'], $element['#field_name'], $element['#type_name'])) {
      $file = (object) $file;
      if ($file->status == FILE_STATUS_PERMANENT) {
        // TODO: We have to add additional code here.
      }
    }
    else {
      form_error($element, t('The file referenced by the %field field does not exist.', array('%field' => $element['#title'])));
    }
    if (empty($element['embedcode']['#value']) && $element['embedcode']['#required']) {
      form_error($element, t('The embed code for the %field field is required.', array('%field' => $element['#title'])));
    }
  }
}

/**
 * FormAPI theme function. Theme the output of an image field.
 */
function theme_embridge_field_widget($element) {
  // Link the label to the upload field.
    $element['#id'] .= '-upload';
  return theme('form_element', $element, $element['#children']);
}

/**
 * Theme the output of widget preview.
 *
 */
function theme_embridge_field_widget_preview($item) {
  // Remove the current description so that we get the filename as the link.
  if (isset($item['data']['description'])) {
    // unset($item['data']['description']);
  }

  $preview = $item['preview_type'] ? $item[$item['preview_type']] : $item['thumbnail'];
  // Add unique timestamp to avoid browser cache.
  $preview .= '?id=' . time();

  return '<div class="embridge-field-file-info">
           <div class="filename"><img src="' . $preview . '" title="' . $item['caption'] . '" alt="' . $item['alttitle'] . '" /></div>
           <div class="filesize">' . format_size($item['filesize']) . '</div>
           <div class="filemime">' . $item['filemime'] . '</div>
         </div>';
}

/**
 * Theme the output of widget item.
 *
 */
function theme_embridge_field_widget_item($element) {
  // Put the upload button directly after the upload field.
  $element['upload']['#field_suffix'] = drupal_render($element['embridge_field_upload']);
  $element['upload']['#theme'] = 'embridge_field_widget_file';

  $output = '';
  $output .= '<div class="embridge-field-element clear-block">';

  if ($element['aid']['#value'] != 0) {
    $output .= '<div class="widget-preview">';
    $output .= drupal_render($element['preview']);
    $output .= '</div>';
  }

  $output .= '<div class="widget-edit">';
  $output .= drupal_render($element);
  $output .= '</div>';
  $output .= '</div>';

  return $output;
}

/**
 * Custom theme function for embridge_field upload elements.
 *
 * This function allows us to put the "Upload" button immediately after the
 * file upload field by respecting the #field_suffix property.
 */
function theme_embridge_field_widget_file($element) {
  $output = '';

  $output .= '<div class="embridge-field-upload clear-block">';

  if (isset($element['#field_prefix'])) {
    $output .= $element['#field_prefix'];
  }

  _form_set_class($element, array('form-file'));
  $output .= '<input type="file" name="' . $element['#name'] . '"' . ($element['#attributes'] ? ' ' . drupal_attributes($element['#attributes']) : '') . ' id="' . $element['#id'] . '" size="' . $element['#size'] . "\" />\n";

  if (isset($element['#field_suffix'])) {
    $output .= $element['#field_suffix'];
  }

  $output .= '</div>';

  return theme('form_element', $element, $output);
}

/**
 * Additional #validate handler for the node form.
 *
 * This function checks the #required properties on file fields and calculates
 * node upload totals for all file fields. The #required property is not
 * properly supported on file fields by Drupal core, so we do this manually.
 */
function embridge_field_node_form_validate($form, &$form_state) {
  foreach ($form['#field_info'] as $field_name => $field) {
    if (!(in_array($field['module'], array('imagefield', 'entermediaasset')))) {
      continue;
    }
    $empty = $field['module'] . '_content_is_empty';
    $valid = FALSE;
    $total_filesize = 0;
    if (!empty($form_state['values'][$field_name])) {
      foreach ($form_state['values'][$field_name] as $delta => $item) {
        if ($empty($item, $field)) {
          continue;
        }
        $valid = TRUE;
        $total_filesize += (int) $item['filesize'];
      }
    }

    if (!$valid && $field['required'] && embridge_field_edit_access($field['type_name'], $field_name)) {
      form_set_error($field_name, t('%title field is required.', array('%title' => $field['widget']['label'])));
    }
    $max_filesize = parse_size($field['widget']['max_filesize_per_node']);
    if ($max_filesize && $total_filesize > $max_filesize) {
      form_set_error($field_name, t('Total filesize for %title, %tsize, exceeds field settings of %msize.',
                                    array(
                                      '%title' => $field['widget']['label'],
                                      '%tsize' => format_size($total_filesize),
                                      '%msize' => format_size($max_filesize),
                                    )
                                  ));
    }
  }
}
